from django.db import models


class DeletedStatusQueryset(models.QuerySet):
    def live(self):
        return self.filter(deleted=False)

    def deleted(self):
        return self.filter(deleted=True)

    def not_deleted(self):
        return self.filter(deleted=False)


class NotDeletedManager(models.Manager):
    def get_queryset(self):
        return DeletedStatusQueryset(self.model, using=self._db)

    def live(self):
        return self.get_queryset().live()

    def deleted(self):
        return self.get_queryset().deleted()


class DeletableManager(models.Manager):
    def get_queryset(self):
        return DeletedStatusQueryset(self.model, using=self._db)

    def live(self):
        return self.get_queryset().not_deleted()

    def not_deleted(self):
        return self.get_queryset().not_deleted()

    def deleted(self):
        return self.get_queryset().deleted()



# Generated by Django 3.2.13 on 2022-05-22 00:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("django_omise", "0002_alter_chargeschedule_card"),
    ]

    operations = [
        migrations.AlterField(
            model_name="source",
            name="type",
            field=models.CharField(
                choices=[
                    ("truemoney", "TrueMoney Wallet"),
                    ("internet_banking_bay", "Krungsri Bank"),
                    ("internet_banking_bbl", "Bangkok Bank"),
                    ("internet_banking_ktb", "Krungthai Bank"),
                    ("internet_banking_scb", "SCB Bank"),
                    ("promptpay", "Promptpay"),
                    ("rabbit_linepay", "Rabbit LINE Pay"),
                ],
                max_length=255,
            ),
        ),
    ]



from .base import OmiseBaseModel, OmiseMetadata, OmiseDeletableModel
from .choices import Currency, OccurrenceStatus, ScheduleStatus, SchedulePeriod

from django.utils.translation import gettext_lazy as _

from django.db import models

from django_omise.omise import omise


class Schedule(OmiseBaseModel, OmiseDeletableModel):

    NON_DEFAULT_FIELDS = OmiseBaseModel.NON_DEFAULT_FIELDS + [
        "charges",
    ]

    omise_class = omise.Schedule

    active = models.BooleanField(
        help_text=_("Whether schedule is status running or expiring.")
    )

    charge = models.OneToOneField(
        "django_omise.ChargeSchedule",
        on_delete=models.PROTECT,
        blank=True,
        null=True,
    )

    end_on = models.DateField(
        blank=True,
        help_text=_("End date of schedule in ISO 8601 format."),
    )

    ended_at = models.CharField(
        max_length=255,
        blank=True,
        help_text=_("Actual end date of schedule in ISO 8601 format."),
    )

    every = models.PositiveIntegerField(
        blank=True,
        null=True,
        help_text=_(
            "How often schedule should run when applied to period. For example, if set to 3 and period is set to week, schedule should run every 3 weeks."
        ),
    )
    in_words = models.TextField(blank=True)
    next_occurrences_on = models.JSONField(blank=True, default=list)

    # Todo # on = models.SomeField
    period = models.CharField(max_length=5, choices=SchedulePeriod.choices, blank=True)

    start_on = models.DateField(
        help_text=_("Start date of schedule in ISO 8601 format.")
    )

    status = models.CharField(
        max_length=50,
        choices=ScheduleStatus.choices,
    )

    # Todo # transfer = models.ForeignKey("django_omise.Transfer")


class Occurrence(OmiseBaseModel):

    message = models.CharField(max_length=255, blank=True)
    processed_at = models.DateTimeField(blank=True, null=True)

    # Todo # result = models.ForeignKey('django_omise.Result')

    retry_on = models.DateField(blank=True, null=True)

    schedule = models.ForeignKey(
        "django_omise.Schedule", on_delete=models.PROTECT, related_name="occurrences"
    )

    schduled_on = models.DateField(null=True, blank=True)

    status = models.CharField(max_length=30, choices=OccurrenceStatus.choices)


class ChargeSchedule(OmiseBaseModel, OmiseMetadata):

    amount = models.IntegerField(
        help_text="Refund amount in smallest unit of charge currency.",
    )

    currency = models.CharField(
        max_length=3,
        choices=Currency.choices,
    )

    card = models.ForeignKey(
        "django_omise.Card",
        on_delete=models.PROTECT,
        related_name="charge_schedules",
        blank=True,
        null=True,
    )

    customer = models.ForeignKey(
        "django_omise.Customer",
        on_delete=models.PROTECT,
        related_name="charge_schedules",
    )

    default_card = models.BooleanField()

    description = models.TextField(blank=True)

    @property
    def human_amount(self) -> float:
        if not self.pk:
            return 0
        if self.currency == Currency.JPY:
            return f"{self.amount:,.2f}"
        return f"{self.amount / 100:,.2f}"



from __future__ import annotations

import omise
import uuid

from django.apps import apps
from django.conf import settings

from typing import Optional, Any, TYPE_CHECKING, List

if TYPE_CHECKING:
    from django.db import models
    from django_omise.models.base import OmiseBaseModel


def get_payment_methods_for_form(payment_methods: Optional[List[str]] = None) -> List[str]:
    """
    Get the payment method as per setting for checkout form

    :returns: List of payment methods by setting OMISE_PAYMENT_METHODS or default to credit card only.
    """
    if payment_methods is None:
        payment_methods = get_payment_methods()

    if "card" in payment_methods:
        payment_methods += ["old_card", "new_card"]

    return payment_methods


def get_payment_methods() -> List[str]:
    """
    Get the payment method as per setting

    :returns: List of payment methods by setting OMISE_PAYMENT_METHODS or default to credit card only.
    """
    payment_methods = setting("OMISE_PAYMENT_METHODS", ["card"])
    return payment_methods


def get_payment_methods_by_country(country: Optional[str] = None) -> List[str]:
    """
    Get the payment methods supported by country

    :param country: 3-digit country code.

    :returns: List of supported payment methods
    """
    payment_methods = [
        "card",
    ]

    if country is None:
        country = setting("OMISE_COUNTRY", "THA")

    if country in ["THA", "JPN"]:
        payment_methods.append("internet_banking")

    return payment_methods


def setting(name: str, default: Optional[Any] = None) -> Optional[Any]:
    """
    Helper function to get a Django setting by name. If setting doesn't exists
    it will return a default.

    :param name: Name of setting
    :param default: Value if setting is unfound
    :returns: Setting's value
    """
    return getattr(settings, name, default)


def is_omise_object_instances(value) -> bool:
    """
    Check if a value is an instance of Omise Classes.

    See all the classes here https://github.com/omise/omise-python/blob/master/omise/__init__.py

    :param value: The value to check.

    :returns: True if value is an instance of Omise Classes, False otherwise
    """
    return isinstance(
        value,
        (
            omise.Account,
            omise.Balance,
            omise.BankAccount,
            omise.Capability,
            omise.Card,
            omise.Chain,
            omise.Charge,
            omise.Collection,
            omise.Customer,
            omise.Dispute,
            omise.Document,
            omise.Event,
            omise.Forex,
            omise.Link,
            omise.Occurrence,
            omise.Receipt,
            omise.Recipient,
            omise.Refund,
            omise.Search,
            omise.Schedule,
            omise.Source,
            omise.Token,
            omise.Transaction,
            omise.Transfer,
        ),
    )


def update_or_create_from_omise_object(
    omise_object: omise.Base,
    raise_if_not_implemented: bool = False,
    ignore_fields: Optional[List[str]] = None,
    uid: Optional[uuid.UUID] = None,
) -> models.Model:
    """
    Update or create a new Django object from Omise object.

    :param omise_object: Any of the Omise object.
    :param raise_if_not_implemented: Whether to raise error if there is no model for Omise object. Default is False.
    :param ignore_fields optional: List of field names to ignore
    :param uuid optional: A unique id for new object.

    :returns: The corresponding model instance.
    """
    model = get_model_from_omise_object(
        omise_object=omise_object, raise_if_not_implemented=raise_if_not_implemented
    )

    if model is None:
        return None

    new_object = model.update_or_create_from_omise_object(
        omise_object=omise_object, ignore_fields=ignore_fields, uid=uid
    )
    return new_object


def get_model_from_omise_object(
    omise_object: omise.Base,
    raise_if_not_implemented: bool = False,
) -> OmiseBaseModel:
    """
    Get the model of the django_omise app with a given Omise object.

    :param omise_object: Any of the Omise object.
    :param raise_if_not_implemented: Whether to raise error if there is no model for Omise object. Default is False.

    :returns: The corresponding model class.
    """
    model_map = {
        "card": get_current_app_model(model_name="Card"),
        "customer": get_current_app_model(model_name="Customer"),
        "charge": get_current_app_model(model_name="Charge"),
        "source": get_current_app_model(model_name="Source"),
        "event": get_current_app_model(model_name="Event"),
        "refund": get_current_app_model(model_name="Refund"),
        "scheduled_charge": get_current_app_model(model_name="ChargeSchedule"),
        "occurrence": get_current_app_model(model_name="Occurrence"),
        "schedule": get_current_app_model(model_name="Schedule"),
    }

    if raise_if_not_implemented and omise_object.object not in model_map:
        raise ValueError(f"The object {omise_object.object} has not been implemented")

    return model_map.get(omise_object.object, None)


def get_current_app_model(model_name: str) -> models.Model:
    """
    Get the model of the django_omise app with a given model name.

    :param model_name str: The model name.

    :returns: The corresponding model class.
    """
    app_label = "django_omise"
    return apps.get_model(app_label=app_label, model_name=model_name)



# Generated by Django 3.2.13 on 2022-05-21 06:27

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_omise', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='chargeschedule',
            name='card',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='charge_schedules', to='django_omise.card'),
        ),
    ]



from __future__ import annotations

import uuid
from django.apps import apps
from django.db import models

from django_omise.omise import omise
from django_omise.utils.core_utils import (
    update_or_create_from_omise_object,
)

from django.utils.translation import gettext_lazy as _

from typing import Optional, Dict
from .managers import DeletableManager

from typing import List


class OmiseMetadata(models.Model):

    metadata = models.JSONField(
        default=dict,
        blank=True,
        help_text=_("Custom metadata for this object."),
    )

    def set_metadata(self, metadata: Optional[Dict] = None) -> "OmiseBaseModel":
        """
        Set the object's medata both on Omise and the Django object.

        :params metadata: A dictionary for medatada

        :returns: Currenct object.
        """

        if metadata is None:
            metadata = {}

        omise_object = self.omise_class.retrieve(self.id)
        omise_object.update(metadata=metadata)

        self.metadata = metadata
        self.save()

        return self

    class Meta:
        abstract = True


class OmiseBaseModel(models.Model):

    id = models.CharField(max_length=255, primary_key=True)
    livemode = models.BooleanField()
    data = models.JSONField(default=dict, blank=True)

    date_created = models.DateTimeField(auto_now_add=True)
    date_updated = models.DateTimeField(auto_now=True)

    uid = models.UUIDField(unique=True, default=uuid.uuid4, editable=False)

    NON_DEFAULT_FIELDS = [
        "id",
        "livemode",
        "date_created",
        "date_updated",
        "data",
        "uid",
    ]

    def __str__(self):
        return f"Omise{self.__class__.__name__}: {self.id}"

    class Meta:
        abstract = True
        ordering = [
            "-date_created",
        ]

    def get_omise_object(self) -> omise.Base:
        """Fetch the object from Omise's server."""
        return self.omise_class.retrieve(self.id)

    def reload_from_omise(self) -> OmiseBaseModel:
        """Reload the data from Omise's server and return the saved object"""
        omise_object = self.omise_class.retrieve(self.id)
        return self.__class__.update_or_create_from_omise_object(
            omise_object=omise_object
        )

    @classmethod
    def update_or_create_from_omise_object(
        cls,
        omise_object: omise.Base,
        ignore_fields: Optional[List[str]] = None,
        uid: Optional[uuid.UUID] = None,
    ) -> OmiseBaseModel:
        """
        Update existing charge or create a new charge from Omise Charge object.

        :param omise_object: An instance of Omise object
        :param ignore_fields optional: List of field names to ignore
        :param uuid optional: A unique id for new object.

        :returns: An instance of current class
        """
        if ignore_fields is None:
            ignore_fields = []

        fields = cls._meta.get_fields()

        defaults = {}

        for field in fields:
            if field.name == "metadata":
                defaults["metadata"] = getattr(
                    omise_object,
                    field.name,
                ).__dict__.get("_attributes")
                continue

            if field.name in cls.NON_DEFAULT_FIELDS:
                continue

            if field.name in ignore_fields:
                continue

            if callable(getattr(omise_object, field.name, None)):
                continue

            if (
                getattr(getattr(omise_object, field.name, None), "object", None)
                == "list"
            ):
                values = getattr(omise_object, field.name)
                for value in values:
                    update_or_create_from_omise_object(omise_object=value)
                continue

            if type(field) in [models.ForeignKey, models.OneToOneField]:
                value = getattr(omise_object, field.name, None)

                if value is None or type(value) is str:
                    defaults[f"{field.name}_id"] = value
                else:

                    new_object = update_or_create_from_omise_object(omise_object=value)

                    defaults[field.name] = new_object

                continue

            if (
                type(field) in [models.TextField, models.CharField]
                and getattr(omise_object, field.name, None) is None
            ):
                defaults[field.name] = ""
                continue

            if (
                getattr(omise_object, field.name, None) is None
                and field.null == False
                and field.default
            ):
                defaults[field.name] = field.default()
                continue

            defaults[field.name] = getattr(omise_object, field.name, None)

        new_object, created = cls.objects.update_or_create(
            pk=omise_object.id,
            livemode=omise_object.livemode,
            defaults=defaults,
        )

        if uid:
            new_object.uid = uid
            new_object.save()

        return new_object


class OmiseDeletableModel(models.Model):
    deleted = models.BooleanField(
        default=False,
        help_text=_("Whether this object is marked as deleted on Omise's server"),
    )

    objects = DeletableManager()

    class Meta:
        abstract = True



from django.db import models
from django.utils.translation import gettext_lazy as _


class ChargeStatus(models.TextChoices):
    FAILED = "failed", _("Failed")
    EXPIRED = "expired", _("Expired")
    PENDING = "pending", _("Pending")
    REVERSED = "reverse", _("Reversed")
    SUCCESSFUL = "successful", _("Successful")
    UNKNOWN = "unknown", _("Unknown")


class Currency(models.TextChoices):
    USD = (
        "USD",
        _("United States Dollar"),
    )
    THB = (
        "THB",
        _("Thai Baht"),
    )
    SGD = (
        "SGD",
        _("Singapore Dollar"),
    )
    JPY = (
        "JPY",
        _("Japanese Yen"),
    )
    GBP = (
        "GBP",
        _("Pound Sterling"),
    )
    EUR = (
        "EUR",
        _("Euro"),
    )
    CNY = (
        "CNY",
        _("Chinese Yuan"),
    )
    AUD = (
        "AUD",
        _("Australian Dollar"),
    )


class ChargeSourceType(models.TextChoices):
    TRUEMONEY_WALLET = "truemoney", _("TrueMoney Wallet")
    INTERNET_BANKING_BAY = "internet_banking_bay", _("Krungsri Bank")
    INTERNET_BANKING_BBL = "internet_banking_bbl", _("Bangkok Bank")
    INTERNET_BANKING_KTB = "internet_banking_ktb", _("Krungthai Bank")
    INTERNET_BANKING_SCB = "internet_banking_scb", _("SCB Bank")
    PROMPTPAY = "promptpay", _("Promptpay")
    RABBIT_LINEPAY = "rabbit_linepay", _("Rabbit LINE Pay")


class SourceFlow(models.TextChoices):
    REDIRECT = "redirect", _("Redirect")
    OFFLINE = "offline", _("Offline")
    APP_REDIRECT = "app_redirect", _("App Redirect")


class ScheduleStatus(models.TextChoices):
    RUNNING = "running", _("Running")
    EXPIRING = "expiring", _("Expiring")
    EXPIRED = "expired", _("Expired")
    DELETED = "deleted", _("Deleted")
    SUSPENDED = "suspended", _("Suspended")


class SchedulePeriod(models.TextChoices):
    DAY = "day", _("Day")
    WEEK = "week", _("Week")
    MONTH = "month", _("Month")


class OccurrenceStatus(models.TextChoices):
    SKIPPED = "skipped", _("Skipped")
    FAILED = "failed", _("Failed")
    SUCCESSFUL = "successful", _("Successful")



from django.urls import path, include

from .views import (
    omise_webhook_view,
    charge_status_json,
    ManagePaymentMethodsView,
    PaymentMethodDeleteView,
    OmiseReturnURIView,
    CheckoutView,
    PromptpayCheckoutView,
    OmiseAccountAndCapabilityJsonView,
)

app_name = "django_omise"

urlpatterns = [
    path(
        "omise_account/",
        OmiseAccountAndCapabilityJsonView.as_view(),
        name="omise_account",
    ),
    path("checkout/", CheckoutView.as_view(), name="checkout"),
    path("return/<uuid:uid>/", OmiseReturnURIView.as_view(), name="return_uri"),
    path("webhook/", omise_webhook_view, name="webhook"),
    path(
        "promptpay_checkout/<pk>/",
        PromptpayCheckoutView.as_view(),
        name="promptpay_checkout",
    ),
    path("charge_status/", charge_status_json, name="charge_status_json"),
    path(
        "payment_methods/",
        include(
            [
                path(
                    "",
                    ManagePaymentMethodsView.as_view(),
                    name="manage_payment_methods",
                ),
                path(
                    "delete/<pk>/",
                    PaymentMethodDeleteView.as_view(),
                    name="delete_payment_method",
                ),
            ]
        ),
    ),
]



from .base import OmiseBaseModel

from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType

from django.db import models

from django_omise.omise import omise

from enum import Enum


class EventType(Enum):
    """
    All Omise event types.

    See each type description here https://www.omise.co/api-webhooks.
    """

    CARD_DESTROY = "card.destroy"
    CARD_UPDATE = "card.update"

    CHARGE_CAPTURE = "charge.capture"
    CHARGE_COMPLETE = "charge.complete"
    CHARGE_CREATE = "charge.create"
    CHARGE_EXPIRE = "charge.expire"
    CHARGE_REVERSE = "charge.reverse"
    CHARGE_UPDATE = "charge.update"

    CUSTOMER_CREATE = "customer.create"
    CUSTOMER_DESTROY = "customer.destroy"
    CUSTOMER_UPDATE = "customer.update"
    CUSTOMER_UPDATE_CARD = "customer.update.card"

    DISPUTE_ACCEPT = "dispute.accept"
    DISPUTE_CLOSE = "dispute.close"
    DISPUTE_CREATE = "dispute.create"
    DISPUTE_UPDATE = "dispute.update"

    LINK_CREATE = "link.create"

    RECIPIENT_ACTIVATE = "recipient.activate"
    RECIPIENT_CREATE = "recipient.create"
    RECIPIENT_DEACTIVATE = "recipient.deactivate"
    RECIPIENT_DESTROY = "recipient.destroy"
    RECIPIENT_UPDATE = "recipient.update"
    RECIPIENT_VERIFY = "recipient.verify"

    REFUND_CREATE = "refund.create"

    SCHEDULE_CREATE = "schedule.create"
    SCHEDULE_DESTROY = "schedule.destroy"
    SCHEDULE_EXPIRE = "schedule.expire"
    SCHEDULE_EXPIRING = "schedule.expiring"
    SCHEDULE_SUSPEND = "schedule.suspend"

    TRANSFER_CREATE = "transfer.create"
    TRANSFER_DESTROY = "transfer.destroy"
    TRANSFER_FAIL = "transfer.fail"
    TRANSFER_PAY = "transfer.pay"
    TRANSFER_SEND = "transfer.send"
    TRANSFER_UPDATE = "transfer.update"


class Event(OmiseBaseModel):

    omise_class = omise.Event

    event_type_choices = [(event.value, event.name) for event in EventType]

    event_type = models.CharField(max_length=50, choices=event_type_choices)

    data = models.JSONField(default=dict)

    content_type = models.ForeignKey(
        ContentType, on_delete=models.CASCADE, blank=True, null=True
    )
    object_id = models.CharField(max_length=255, blank=True, null=True)
    event_object = GenericForeignKey("content_type", "object_id")

    class Meta:
        indexes = [
            models.Index(fields=["content_type", "object_id"]),
        ]



# Generated by Django 3.2.13 on 2022-05-21 05:57

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Card",
            fields=[
                (
                    "id",
                    models.CharField(max_length=255, primary_key=True, serialize=False),
                ),
                ("livemode", models.BooleanField()),
                ("data", models.JSONField(blank=True, default=dict)),
                ("date_created", models.DateTimeField(auto_now_add=True)),
                ("date_updated", models.DateTimeField(auto_now=True)),
                (
                    "uid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                ("name", models.CharField(blank=True, max_length=255)),
                ("phone_number", models.CharField(blank=True, max_length=255)),
                ("postal_code", models.CharField(blank=True, max_length=255)),
                ("state", models.CharField(blank=True, max_length=255)),
                ("street1", models.CharField(blank=True, max_length=255)),
                ("street2", models.CharField(blank=True, max_length=255)),
                ("tokenization_method", models.CharField(blank=True, max_length=255)),
                ("first_digits", models.CharField(blank=True, max_length=6)),
                ("last_digits", models.CharField(max_length=4)),
                ("bank", models.CharField(blank=True, max_length=255)),
                ("brand", models.CharField(blank=True, max_length=255)),
                ("city", models.CharField(blank=True, max_length=255)),
                ("country", models.CharField(blank=True, max_length=255)),
                ("expiration_month", models.CharField(blank=True, max_length=2)),
                ("expiration_year", models.CharField(blank=True, max_length=4)),
                ("financing", models.CharField(blank=True, max_length=10)),
                (
                    "deleted",
                    models.BooleanField(
                        default=False, help_text="Whethe this card was deleted."
                    ),
                ),
            ],
            options={
                "ordering": ["-date_created"],
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Source",
            fields=[
                (
                    "id",
                    models.CharField(max_length=255, primary_key=True, serialize=False),
                ),
                ("livemode", models.BooleanField()),
                ("data", models.JSONField(blank=True, default=dict)),
                ("date_created", models.DateTimeField(auto_now_add=True)),
                ("date_updated", models.DateTimeField(auto_now=True)),
                (
                    "uid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                (
                    "amount",
                    models.IntegerField(
                        help_text="Source amount in smallest unit of source currency"
                    ),
                ),
                ("bank", models.CharField(blank=True, max_length=255)),
                (
                    "barcode",
                    models.CharField(
                        blank=True,
                        help_text="Source amount in smallest unit of source currency",
                        max_length=255,
                    ),
                ),
                (
                    "charge_status",
                    models.CharField(
                        choices=[
                            ("failed", "Failed"),
                            ("expired", "Expired"),
                            ("pending", "Pending"),
                            ("reverse", "Reversed"),
                            ("successful", "Successful"),
                            ("unknown", "Unknown"),
                        ],
                        help_text="Status of charge created using this source (if any)",
                        max_length=10,
                    ),
                ),
                (
                    "currency",
                    models.CharField(
                        choices=[
                            ("USD", "United States Dollar"),
                            ("THB", "Thai Baht"),
                            ("SGD", "Singapore Dollar"),
                            ("JPY", "Japanese Yen"),
                            ("GBP", "Pound Sterling"),
                            ("EUR", "Euro"),
                            ("CNY", "Chinese Yuan"),
                            ("AUD", "Australian Dollar"),
                        ],
                        max_length=3,
                    ),
                ),
                (
                    "email",
                    models.CharField(
                        blank=True, help_text="The payer's email", max_length=255
                    ),
                ),
                (
                    "flow",
                    models.CharField(
                        choices=[
                            ("redirect", "Redirect"),
                            ("offline", "Offline"),
                            ("app_redirect", "App Redirect"),
                        ],
                        help_text="The payment flow payers need to go through to complete the payment. One of redirect, offline, or app_redirect",
                        max_length=50,
                    ),
                ),
                (
                    "installment_term",
                    models.IntegerField(
                        blank=True, help_text="Installment term in months", null=True
                    ),
                ),
                ("mobile_number", models.CharField(blank=True, max_length=50)),
                (
                    "name",
                    models.CharField(
                        blank=True, help_text="Payer's name", max_length=255
                    ),
                ),
                ("phone_number", models.CharField(blank=True, max_length=50)),
                (
                    "platform_type",
                    models.CharField(
                        blank=True,
                        help_text="Platform from which payer is making a payment. One of WEB, IOS, or ANDROID",
                        max_length=50,
                    ),
                ),
                ("receipt_amount", models.IntegerField(blank=True, null=True)),
                ("store_id", models.CharField(blank=True, max_length=255)),
                ("store_name", models.CharField(blank=True, max_length=255)),
                ("terminal_id", models.CharField(blank=True, max_length=255)),
                (
                    "type",
                    models.CharField(
                        choices=[
                            ("truemoney", "TrueMoney Wallet"),
                            ("internet_banking_bay", "Krungsri Bank"),
                            ("internet_banking_bbl", "Bangkok Bank"),
                            ("internet_banking_ktb", "Krungthai Bank"),
                            ("internet_banking_scb", "SCB Bank"),
                            ("promptpay", "Promptpay"),
                        ],
                        max_length=255,
                    ),
                ),
                (
                    "zero_interest_installments",
                    models.BooleanField(blank=True, null=True),
                ),
            ],
            options={
                "ordering": ["-date_created"],
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Event",
            fields=[
                (
                    "id",
                    models.CharField(max_length=255, primary_key=True, serialize=False),
                ),
                ("livemode", models.BooleanField()),
                ("date_created", models.DateTimeField(auto_now_add=True)),
                ("date_updated", models.DateTimeField(auto_now=True)),
                (
                    "uid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                (
                    "event_type",
                    models.CharField(
                        choices=[
                            ("card.destroy", "CARD_DESTROY"),
                            ("card.update", "CARD_UPDATE"),
                            ("charge.capture", "CHARGE_CAPTURE"),
                            ("charge.complete", "CHARGE_COMPLETE"),
                            ("charge.create", "CHARGE_CREATE"),
                            ("charge.expire", "CHARGE_EXPIRE"),
                            ("charge.reverse", "CHARGE_REVERSE"),
                            ("charge.update", "CHARGE_UPDATE"),
                            ("customer.create", "CUSTOMER_CREATE"),
                            ("customer.destroy", "CUSTOMER_DESTROY"),
                            ("customer.update", "CUSTOMER_UPDATE"),
                            ("customer.update.card", "CUSTOMER_UPDATE_CARD"),
                            ("dispute.accept", "DISPUTE_ACCEPT"),
                            ("dispute.close", "DISPUTE_CLOSE"),
                            ("dispute.create", "DISPUTE_CREATE"),
                            ("dispute.update", "DISPUTE_UPDATE"),
                            ("link.create", "LINK_CREATE"),
                            ("recipient.activate", "RECIPIENT_ACTIVATE"),
                            ("recipient.create", "RECIPIENT_CREATE"),
                            ("recipient.deactivate", "RECIPIENT_DEACTIVATE"),
                            ("recipient.destroy", "RECIPIENT_DESTROY"),
                            ("recipient.update", "RECIPIENT_UPDATE"),
                            ("recipient.verify", "RECIPIENT_VERIFY"),
                            ("refund.create", "REFUND_CREATE"),
                            ("schedule.create", "SCHEDULE_CREATE"),
                            ("schedule.destroy", "SCHEDULE_DESTROY"),
                            ("schedule.expire", "SCHEDULE_EXPIRE"),
                            ("schedule.expiring", "SCHEDULE_EXPIRING"),
                            ("schedule.suspend", "SCHEDULE_SUSPEND"),
                            ("transfer.create", "TRANSFER_CREATE"),
                            ("transfer.destroy", "TRANSFER_DESTROY"),
                            ("transfer.fail", "TRANSFER_FAIL"),
                            ("transfer.pay", "TRANSFER_PAY"),
                            ("transfer.send", "TRANSFER_SEND"),
                            ("transfer.update", "TRANSFER_UPDATE"),
                        ],
                        max_length=50,
                    ),
                ),
                ("data", models.JSONField(default=dict)),
                ("object_id", models.CharField(blank=True, max_length=255, null=True)),
                (
                    "content_type",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.contenttype",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="Customer",
            fields=[
                (
                    "id",
                    models.CharField(max_length=255, primary_key=True, serialize=False),
                ),
                ("livemode", models.BooleanField()),
                ("data", models.JSONField(blank=True, default=dict)),
                ("date_created", models.DateTimeField(auto_now_add=True)),
                ("date_updated", models.DateTimeField(auto_now=True)),
                (
                    "uid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                (
                    "deleted",
                    models.BooleanField(
                        default=False, help_text="Whether this customer was deleted."
                    ),
                ),
                (
                    "user",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        related_name="omise_customers",
                        to=settings.AUTH_USER_MODEL,
                    ),
                ),
            ],
            options={
                "ordering": ["-date_created"],
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Charge",
            fields=[
                (
                    "id",
                    models.CharField(max_length=255, primary_key=True, serialize=False),
                ),
                ("livemode", models.BooleanField()),
                ("data", models.JSONField(blank=True, default=dict)),
                ("date_created", models.DateTimeField(auto_now_add=True)),
                ("date_updated", models.DateTimeField(auto_now=True)),
                (
                    "uid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("failed", "Failed"),
                            ("expired", "Expired"),
                            ("pending", "Pending"),
                            ("reverse", "Reversed"),
                            ("successful", "Successful"),
                            ("unknown", "Unknown"),
                        ],
                        help_text="Status of this charge.",
                        max_length=10,
                    ),
                ),
                (
                    "amount",
                    models.IntegerField(
                        help_text="Charge amount in the smallest unit of charge currency"
                    ),
                ),
                (
                    "authorize_uri",
                    models.CharField(
                        blank=True,
                        help_text="URI for payment authorization (e.g. 3-D Secure).",
                        max_length=255,
                    ),
                ),
                (
                    "authorized",
                    models.BooleanField(
                        blank=True,
                        help_text="Whether charge has been authorized.",
                        null=True,
                    ),
                ),
                (
                    "capturable",
                    models.BooleanField(
                        blank=True,
                        help_text="Whether charge is able to be captured (paid).",
                        null=True,
                    ),
                ),
                (
                    "capture",
                    models.BooleanField(
                        blank=True,
                        help_text="Whether charge is set to be automatically captured (paid).",
                        null=True,
                    ),
                ),
                (
                    "currency",
                    models.CharField(
                        choices=[
                            ("USD", "United States Dollar"),
                            ("THB", "Thai Baht"),
                            ("SGD", "Singapore Dollar"),
                            ("JPY", "Japanese Yen"),
                            ("GBP", "Pound Sterling"),
                            ("EUR", "Euro"),
                            ("CNY", "Chinese Yuan"),
                            ("AUD", "Australian Dollar"),
                        ],
                        max_length=3,
                    ),
                ),
                ("description", models.TextField(blank=True)),
                ("disputable", models.BooleanField(blank=True, null=True)),
                (
                    "expired",
                    models.BooleanField(
                        blank=True,
                        help_text="Whether this charge has expired.",
                        null=True,
                    ),
                ),
                ("expired_at", models.DateTimeField(blank=True, null=True)),
                ("expires_at", models.DateTimeField(blank=True, null=True)),
                ("failure_code", models.CharField(blank=True, max_length=255)),
                ("failure_message", models.TextField(blank=True)),
                ("fee", models.IntegerField()),
                ("fee_vat", models.IntegerField()),
                (
                    "funding_amount",
                    models.IntegerField(
                        blank=True,
                        help_text="For multi-currency charges, amount after exchange into account funding currency.",
                        null=True,
                    ),
                ),
                (
                    "funding_currency",
                    models.CharField(
                        choices=[
                            ("USD", "United States Dollar"),
                            ("THB", "Thai Baht"),
                            ("SGD", "Singapore Dollar"),
                            ("JPY", "Japanese Yen"),
                            ("GBP", "Pound Sterling"),
                            ("EUR", "Euro"),
                            ("CNY", "Chinese Yuan"),
                            ("AUD", "Australian Dollar"),
                        ],
                        max_length=3,
                    ),
                ),
                (
                    "interest",
                    models.IntegerField(
                        blank=True,
                        help_text="Interest paid by customer or merchant over the course of an installment term.",
                        null=True,
                    ),
                ),
                (
                    "interest_vat",
                    models.IntegerField(
                        blank=True,
                        help_text="Value-added Tax (VAT) applied to interest.",
                        null=True,
                    ),
                ),
                (
                    "ip",
                    models.CharField(
                        blank=True,
                        help_text="IP address provided to Omise at charge creation. May be IPv4 or IPv6.",
                        max_length=255,
                    ),
                ),
                (
                    "metadata",
                    models.JSONField(
                        blank=True,
                        default=dict,
                        help_text="Custom metadata for this charge.",
                    ),
                ),
                (
                    "net",
                    models.IntegerField(
                        help_text="funding_amount after fees, interest, and VAT deducted."
                    ),
                ),
                (
                    "paid",
                    models.BooleanField(
                        help_text="Whether charge has been captured (paid)."
                    ),
                ),
                ("paid_at", models.DateTimeField(blank=True, null=True)),
                (
                    "refundable",
                    models.BooleanField(
                        help_text="Whether charge is able to be refunded."
                    ),
                ),
                (
                    "refunded_amount",
                    models.IntegerField(
                        blank=True,
                        help_text="Refunded amount in smallest unit of currency.",
                        null=True,
                    ),
                ),
                (
                    "return_uri",
                    models.TextField(
                        blank=True,
                        help_text="URI to which customer is redirected after 3-D Secure card authorization or other redirect-based authorization.",
                    ),
                ),
                (
                    "reversed",
                    models.BooleanField(
                        blank=True,
                        help_text="Whether charge authorization is able to be reversed.",
                        null=True,
                    ),
                ),
                (
                    "voided",
                    models.BooleanField(
                        blank=True,
                        help_text="Whether, in the case of a refund, charge was voided instead. Charges are voided if refund is processed before settlement time.",
                        null=True,
                    ),
                ),
                (
                    "zero_interest_installments",
                    models.BooleanField(
                        blank=True,
                        help_text="Whether merchant absorbs the interest for installment payments; must match value in associated source.",
                        null=True,
                    ),
                ),
                (
                    "card",
                    models.ForeignKey(
                        blank=True,
                        help_text="Card that was charged (if card was charged).",
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="charges",
                        to="django_omise.card",
                    ),
                ),
                (
                    "customer",
                    models.ForeignKey(
                        blank=True,
                        help_text="Customer associated with charge.",
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="charges",
                        to="django_omise.customer",
                    ),
                ),
                (
                    "source",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="charges",
                        to="django_omise.source",
                    ),
                ),
            ],
            options={
                "ordering": ["-date_created"],
                "abstract": False,
            },
        ),
        migrations.AddField(
            model_name="card",
            name="customer",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="cards",
                to="django_omise.customer",
            ),
        ),
        migrations.AddIndex(
            model_name="event",
            index=models.Index(
                fields=["content_type", "object_id"],
                name="django_omis_content_80fefa_idx",
            ),
        ),
        migrations.AddField(
            model_name="card",
            name="fingerprint",
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name="customer",
            name="default_card",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                related_name="+",
                to="django_omise.card",
            ),
        ),
        migrations.AddField(
            model_name="customer",
            name="description",
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name="customer",
            name="email",
            field=models.EmailField(blank=True, max_length=254),
        ),
        migrations.AlterField(
            model_name="charge",
            name="metadata",
            field=models.JSONField(
                blank=True, default=dict, help_text="Custom metadata for this object."
            ),
        ),
        migrations.AddField(
            model_name="customer",
            name="metadata",
            field=models.JSONField(
                blank=True, default=dict, help_text="Custom metadata for this object."
            ),
        ),
        migrations.CreateModel(
            name="Refund",
            fields=[
                (
                    "id",
                    models.CharField(max_length=255, primary_key=True, serialize=False),
                ),
                ("livemode", models.BooleanField()),
                ("data", models.JSONField(blank=True, default=dict)),
                ("date_created", models.DateTimeField(auto_now_add=True)),
                ("date_updated", models.DateTimeField(auto_now=True)),
                (
                    "uid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                (
                    "amount",
                    models.IntegerField(
                        help_text="Refund amount in smallest unit of charge currency."
                    ),
                ),
                (
                    "currency",
                    models.CharField(
                        choices=[
                            ("USD", "United States Dollar"),
                            ("THB", "Thai Baht"),
                            ("SGD", "Singapore Dollar"),
                            ("JPY", "Japanese Yen"),
                            ("GBP", "Pound Sterling"),
                            ("EUR", "Euro"),
                            ("CNY", "Chinese Yuan"),
                            ("AUD", "Australian Dollar"),
                        ],
                        max_length=3,
                    ),
                ),
                (
                    "funding_amount",
                    models.IntegerField(
                        blank=True,
                        help_text="For multi-currency charges, amount after exchange into account funding currency.",
                        null=True,
                    ),
                ),
                (
                    "funding_currency",
                    models.CharField(
                        choices=[
                            ("USD", "United States Dollar"),
                            ("THB", "Thai Baht"),
                            ("SGD", "Singapore Dollar"),
                            ("JPY", "Japanese Yen"),
                            ("GBP", "Pound Sterling"),
                            ("EUR", "Euro"),
                            ("CNY", "Chinese Yuan"),
                            ("AUD", "Australian Dollar"),
                        ],
                        max_length=3,
                    ),
                ),
                (
                    "metadata",
                    models.JSONField(
                        blank=True,
                        default=dict,
                        help_text="Custom metadata for this object.",
                    ),
                ),
                ("voided", models.BooleanField()),
                (
                    "charge",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="refunds",
                        to="django_omise.charge",
                    ),
                ),
            ],
            options={
                "ordering": ["-date_created"],
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="ChargeSchedule",
            fields=[
                (
                    "metadata",
                    models.JSONField(
                        blank=True,
                        default=dict,
                        help_text="Custom metadata for this object.",
                    ),
                ),
                (
                    "id",
                    models.CharField(max_length=255, primary_key=True, serialize=False),
                ),
                ("livemode", models.BooleanField()),
                ("data", models.JSONField(blank=True, default=dict)),
                ("date_created", models.DateTimeField(auto_now_add=True)),
                ("date_updated", models.DateTimeField(auto_now=True)),
                (
                    "uid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                (
                    "amount",
                    models.IntegerField(
                        help_text="Refund amount in smallest unit of charge currency."
                    ),
                ),
                (
                    "currency",
                    models.CharField(
                        choices=[
                            ("USD", "United States Dollar"),
                            ("THB", "Thai Baht"),
                            ("SGD", "Singapore Dollar"),
                            ("JPY", "Japanese Yen"),
                            ("GBP", "Pound Sterling"),
                            ("EUR", "Euro"),
                            ("CNY", "Chinese Yuan"),
                            ("AUD", "Australian Dollar"),
                        ],
                        max_length=3,
                    ),
                ),
                ("default_card", models.BooleanField()),
                ("description", models.TextField(blank=True)),
                (
                    "card",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="charge_schedules",
                        to="django_omise.card",
                    ),
                ),
                (
                    "customer",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="charge_schedules",
                        to="django_omise.customer",
                    ),
                ),
            ],
            options={
                "ordering": ["-date_created"],
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Schedule",
            fields=[
                (
                    "id",
                    models.CharField(max_length=255, primary_key=True, serialize=False),
                ),
                ("livemode", models.BooleanField()),
                ("data", models.JSONField(blank=True, default=dict)),
                ("date_created", models.DateTimeField(auto_now_add=True)),
                ("date_updated", models.DateTimeField(auto_now=True)),
                (
                    "uid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                (
                    "deleted",
                    models.BooleanField(
                        default=False,
                        help_text="Whether this object is marked as deleted on Omise's server",
                    ),
                ),
                (
                    "active",
                    models.BooleanField(
                        help_text="Whether schedule is status running or expiring."
                    ),
                ),
                (
                    "end_on",
                    models.DateField(
                        blank=True, help_text="End date of schedule in ISO 8601 format."
                    ),
                ),
                (
                    "ended_at",
                    models.CharField(
                        blank=True,
                        help_text="Actual end date of schedule in ISO 8601 format.",
                        max_length=255,
                    ),
                ),
                (
                    "every",
                    models.PositiveIntegerField(
                        blank=True,
                        help_text="How often schedule should run when applied to period. For example, if set to 3 and period is set to week, schedule should run every 3 weeks.",
                        null=True,
                    ),
                ),
                ("in_words", models.TextField(blank=True)),
                ("next_occurrences_on", models.JSONField(blank=True, default=list)),
                (
                    "period",
                    models.CharField(
                        blank=True,
                        choices=[("day", "Day"), ("week", "Week"), ("month", "Month")],
                        max_length=5,
                    ),
                ),
                (
                    "start_on",
                    models.DateField(
                        help_text="Start date of schedule in ISO 8601 format."
                    ),
                ),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("running", "Running"),
                            ("expiring", "Expiring"),
                            ("expired", "Expired"),
                            ("deleted", "Deleted"),
                            ("suspended", "Suspended"),
                        ],
                        max_length=50,
                    ),
                ),
                (
                    "charge",
                    models.OneToOneField(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.PROTECT,
                        to="django_omise.chargeschedule",
                    ),
                ),
            ],
            options={
                "ordering": ["-date_created"],
                "abstract": False,
            },
        ),
        migrations.CreateModel(
            name="Occurrence",
            fields=[
                (
                    "id",
                    models.CharField(max_length=255, primary_key=True, serialize=False),
                ),
                ("livemode", models.BooleanField()),
                ("data", models.JSONField(blank=True, default=dict)),
                ("date_created", models.DateTimeField(auto_now_add=True)),
                ("date_updated", models.DateTimeField(auto_now=True)),
                (
                    "uid",
                    models.UUIDField(default=uuid.uuid4, editable=False, unique=True),
                ),
                ("message", models.CharField(blank=True, max_length=255)),
                ("processed_at", models.DateTimeField(blank=True, null=True)),
                ("retry_on", models.DateField(blank=True, null=True)),
                ("schduled_on", models.DateField(blank=True, null=True)),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("skipped", "Skipped"),
                            ("failed", "Failed"),
                            ("successful", "Successful"),
                        ],
                        max_length=30,
                    ),
                ),
                (
                    "schedule",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        related_name="occurrences",
                        to="django_omise.schedule",
                    ),
                ),
            ],
            options={
                "ordering": ["-date_created"],
                "abstract": False,
            },
        ),
        migrations.AddField(
            model_name="charge",
            name="schedule",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.PROTECT,
                related_name="charges",
                to="django_omise.schedule",
            ),
        ),
    ]



from django import forms
from django.conf import settings

from .models.core import Customer, Card
from .models.choices import Currency, ChargeSourceType
from .omise import omise
from .utils.core_utils import get_payment_methods_for_form

from django.utils.translation import gettext_lazy as _

from typing import Optional, List, Dict


class AddCardForm(forms.Form):

    omise_token = forms.CharField(
        widget=forms.HiddenInput,
    )

    omise_public_key = forms.CharField(
        widget=forms.HiddenInput,
        disabled=True,
        initial=settings.OMISE_PUBLIC_KEY,
    )

    card_number = forms.CharField(
        required=False,
    )

    name_on_card = forms.CharField(
        required=False,
    )

    expiration_month = forms.CharField(
        required=False,
    )

    expiration_year = forms.CharField(
        required=False,
    )

    security_number = forms.CharField(
        required=False,
    )

    class Media:
        js = (
            "django_omise/js/jquery.min.js",
            "django_omise/js/jquery.payments.js",
            "django_omise/js/omise.js",
            "django_omise/js/add_card.js",
        )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["card_number"].widget.attrs["placeholder"] = "1234 1234 1234 1234"
        self.fields["name_on_card"].widget.attrs["placeholder"] = "John Doe"
        self.fields["expiration_month"].widget.attrs["placeholder"] = "1-12"
        self.fields["expiration_year"].widget.attrs["placeholder"] = "2030"
        self.fields["security_number"].widget.attrs["placeholder"] = "***"

    def clean_omise_token(self) -> omise.Token:
        """
        Check that the credit card token is valid.
        """
        try:
            omise_token = omise.Token.retrieve(self.cleaned_data["omise_token"])
        except omise.errors.NotFoundError:
            raise forms.ValidationError(
                _("Error processing the card. Please try again.")
            )

        return omise_token

    def add_card_to_user(self, customer: "Customer"):
        """
        Add the new card to the user.

        :params customer: The customer instance

        :returns None:
        """
        customer.add_card(self.cleaned_data["omise_token"])


class CheckoutWithCardsForm(forms.Form):

    card = forms.ModelChoiceField(queryset=None, widget=forms.RadioSelect)

    def __init__(self, *args, **kwargs):

        user = kwargs.pop("user", None)

        super().__init__(*args, **kwargs)

        self.user = user

        if user is None:
            return

        if not user.is_authenticated:
            return

        self.omise_customer = self.user.extension.omise_customer
        self.fields["card"].queryset = self.omise_customer.cards.live()


class PayWithNewCardForm(forms.Form):

    omise_token = forms.CharField(
        widget=forms.HiddenInput,
        required=False,
    )

    card_number = forms.CharField(
        required=False,
    )

    name_on_card = forms.CharField(
        required=False,
    )

    expiration_month = forms.CharField(
        required=False,
    )

    expiration_year = forms.CharField(
        required=False,
    )

    security_number = forms.CharField(
        required=False,
    )

    keep_card = forms.BooleanField(initial=False, required=False)

    class Media:
        js = (
            "django_omise/js/jquery.min.js",
            "django_omise/js/jquery.payments.js",
            "django_omise/js/omise.js",
            "django_omise/js/checkout.js",
        )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields["card_number"].widget.attrs["placeholder"] = "1234 1234 1234 1234"
        self.fields["name_on_card"].widget.attrs["placeholder"] = "John Doe"
        self.fields["expiration_month"].widget.attrs["placeholder"] = "1-12"
        self.fields["expiration_year"].widget.attrs["placeholder"] = "2030"
        self.fields["security_number"].widget.attrs["placeholder"] = "***"

        if settings.DEBUG and settings.OMISE_LIVE_MODE is False:
            self.fields["card_number"].initial = "4111 1111 1111 1111"
            self.fields["name_on_card"].initial = "James"
            self.fields["expiration_month"].initial = "10"
            self.fields["expiration_year"].initial = "2030"
            self.fields["security_number"].initial = "123"

    def clean_omise_token(self) -> omise.Token:
        """
        Check that the credit card token is valid.
        """
        if not self.cleaned_data["omise_token"]:
            return None

        try:
            omise_token = omise.Token.retrieve(self.cleaned_data["omise_token"])
        except omise.errors.NotFoundError:
            raise forms.ValidationError(
                _("Error processing the card. Please try again.")
            )

        return omise_token

    def add_card_to_user(self, customer: "Customer"):
        """
        Add the new card to the user.

        :params customer: The customer instance

        :returns None:
        """
        card = customer.add_card(self.cleaned_data["omise_token"])
        return card


class PayWithExistingCardForm(forms.Form):

    card = forms.ModelChoiceField(
        queryset=None, widget=forms.RadioSelect, required=False
    )


class PayWithInternetBankingForm(forms.Form):

    bank = forms.ChoiceField(
        choices=(
            (ChargeSourceType.INTERNET_BANKING_BAY, _("Krungsri Bank")),
            (ChargeSourceType.INTERNET_BANKING_BBL, _("Bangkok Bank")),
            (ChargeSourceType.INTERNET_BANKING_KTB, _("Krungthai Bank")),
            (ChargeSourceType.INTERNET_BANKING_SCB, _("SCB Bank")),
        ),
        widget=forms.RadioSelect,
        required=False,
        initial="internet_banking_bay",
    )


class PayWithTrueMoneyForm(forms.Form):
    phone_number = forms.CharField(max_length=10, required=False)


class CheckoutForm(
    PayWithExistingCardForm,
    PayWithNewCardForm,
    PayWithInternetBankingForm,
    PayWithTrueMoneyForm,
):

    payment_method = forms.ChoiceField(
        choices=(), required=True, widget=forms.RadioSelect, initial="new_card"
    )

    omise_public_key = forms.CharField(
        widget=forms.HiddenInput,
        disabled=True,
        initial=settings.OMISE_PUBLIC_KEY,
    )

    def __init__(
        self,
        user,
        amount: int,
        currency: Currency,
        payment_methods: Optional[List[str]] = None,
        *args,
        **kwargs
    ):
        super().__init__(*args, **kwargs)

        self.user = user
        self.amount = amount
        self.currency = currency

        all_payment_methods = {
            "old_card": _("Pay with your card"),
            "new_card": _("Pay with a new card"),
            "internet_banking": _("Internet banking"),
            "truemoney_wallet": _("TrueMoney Wallet"),
            ChargeSourceType.PROMPTPAY: ChargeSourceType.PROMPTPAY.label,
            ChargeSourceType.RABBIT_LINEPAY: ChargeSourceType.RABBIT_LINEPAY.label,
        }

        payment_methods = {
            key: value
            for key, value in all_payment_methods.items()
            if key in get_payment_methods_for_form(payment_methods)
        }

        if user is None or user.is_authenticated == False:

            del self.fields["keep_card"]
            del self.fields["card"]

            del payment_methods["old_card"]

        else:

            self.omise_customer, created = Customer.get_or_create(user=user)

            if self.omise_customer.cards.live().exists():

                self.fields["payment_method"].initial = "old_card"
                self.fields["card"].queryset = self.omise_customer.cards.live()
                self.fields["card"].initial = self.omise_customer.cards.live().first()

            else:

                del payment_methods["old_card"]
                del self.fields["card"]

        payment_method_choices = []
        for key, value in payment_methods.items():
            payment_method_choices.append((key, value))
        self.fields["payment_method"].choices = payment_method_choices

    def clean(self):
        cleaned_data = super().clean()
        payment_method = cleaned_data["payment_method"]
        omise_token = cleaned_data.get("omise_token")
        card = cleaned_data.get("card")
        bank = cleaned_data.get("bank")

        if payment_method == "new_card" and omise_token is None:
            raise forms.ValidationError(
                {"card_number": _("You need to provide a new card details")}
            )

        if payment_method == "old_card" and card is None:
            raise forms.ValidationError(
                {"card": _("You need to select at least one card")}
            )

        if payment_method == "internet_banking" and not bank:
            raise forms.ValidationError(
                {"bank": _("You need to select at least one bank")}
            )

        phone_number = cleaned_data.get("phone_number")
        if payment_method == "truemoney_wallet" and not phone_number:
            raise forms.ValidationError(
                {
                    "phone_number": _(
                        "Mobile number is required for TrueMoney Wallet payment"
                    )
                }
            )

    def get_charge_type_details(self) -> Dict:
        """
        Build the charge type details from selected payment method and options

        :returns: Dict of charge type details.
                    If card is charged, it will be {'card': core.Card}
                    if token is charged, it will be {'token': omise.Token}
                    If source is charged, it will be {'source': {'type': source_type_string, ...}}
        """

        payment_method = self.cleaned_data["payment_method"]
        charge_details = {}

        if payment_method == "new_card":

            if self.cleaned_data.get("keep_card", None) == True:
                card = self.add_card_to_user()
                charge_details["card"] = card
            else:
                charge_details["token"] = self.cleaned_data["omise_token"]

        if payment_method == "old_card":
            charge_details["card"] = self.cleaned_data["card"]

        if payment_method == "internet_banking":
            charge_details["source"] = {"type": self.cleaned_data["bank"]}

        if payment_method == "truemoney_wallet":
            charge_details["source"] = {
                "type": ChargeSourceType.TRUEMONEY_WALLET,
                "phone_number": self.cleaned_data["phone_number"],
            }

        if payment_method in [
            ChargeSourceType.PROMPTPAY,
            ChargeSourceType.RABBIT_LINEPAY,
        ]:
            charge_details["source"] = {"type": payment_method}

        return charge_details

    def add_card_to_user(self) -> Card:
        """
        Add the new card to the user.

        :params customer: The customer instance

        :returns: The new Card instance
        """
        return self.omise_customer.add_card(self.cleaned_data["omise_token"])

    @property
    def new_card_fields(self) -> List[forms.fields.Field]:

        new_card_field_names = [
            "omise_token",
            "card_number",
            "name_on_card",
            "expiration_year",
            "expiration_month",
            "security_number",
            "keep_card",
        ]

        return [
            field
            for field in self
            if field.name in new_card_field_names and not field.is_hidden
        ]

    @property
    def truemoney_fields(self) -> List[forms.fields.Field]:

        fields = [
            "phone_number",
        ]

        return [field for field in self if field.name in fields and not field.is_hidden]

    @property
    def human_amount(self) -> float:
        if self.currency.upper() == Currency.JPY:
            return float(self.amount)

        return self.amount / 100
